/* ============================================================
 *
 * This file is part of the RSB project
 *
 * Copyright (C) 2011 Johannes Wienke
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#pragma once

#include <rsc/threading/SynchronizedQueue.h>

#include <rsb/Event.h>
#include <rsb/Handler.h>

class EventQueuePushHandler: public rsb::Handler {
private:
    boost::shared_ptr<rsc::threading::SynchronizedQueue<rsb::EventPtr> > queue;
public:

    /**
     * Constructs a new instance.
     *
     * @param queue the queue to push received data on
     * @param method method of this handler to react on, empty means all events
     */
    EventQueuePushHandler(
	boost::shared_ptr<rsc::threading::SynchronizedQueue<rsb::EventPtr> > queue,
	const std::string &method = "") :
	Handler(method), queue(queue) {
    }

    std::string getClassName() const {
	return "EventQueuePushHandler";
    }

    void printContents(std::ostream &stream) const {
	stream << "queue = " << queue;
    }

    void handle(rsb::EventPtr event) {
	queue->push(event);
    }

};
