/* ============================================================
 *
 * This file is part of the RSB project
 *
 * Copyright (C) 2011 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include "NaoAudioImporter.h"

#include <stdlib.h>
#include <sys/time.h>
#include <math.h>

#include <iostream>

#include <boost/thread.hpp>
#include <boost/timer.hpp>

#include <gst/gst.h>

#include <rsb/Factory.h>
#include <rsb/QueuePushHandler.h>
#include <rsb/converter/Repository.h>
#include <rsb/converter/ProtocolBufferConverter.h>

#include "EventQueuePushHandler.h"

using namespace std;

using namespace boost;

using namespace rsb;
using namespace rsb::converter;

// Constructor
NaoAudioImporter::NaoAudioImporter() :
    queue(new rsc::threading::SynchronizedQueue<EventPtr>(1)),
    firstTimestamp(0), inputRemaining(0) {

    shared_ptr<ProtocolBufferConverter<rst::SoundMessage> > converter(
	new ProtocolBufferConverter<rst::SoundMessage> ());
    cout << "Registering converter for wire schema "
	 << converter->getWireSchema() << endl;
    stringConverterRepository()->registerConverter(converter);

    // You can do this right here, but then the Listener won't get the ScopeName in time.
    // See: gstimport.ccp gst_rsb_import_set_property
    /*
      Factory &factory = Factory::getInstance();
      listener = factory.createListener(Scope("/foo/bla"));
      listener->addHandler(HandlerPtr(new QueuePushHandler<GSimage> (queue)));
    */

    count = 0;
    gettimeofday(&start, 0);
    go = false;
}

// Destructor
NaoAudioImporter::~NaoAudioImporter() {
}

void NaoAudioImporter::setScopeName(const gchar *scope) {
	g_print("Scope: %s \n", scope);
	this->scopename = std::string(scope);
}

void NaoAudioImporter::setTime(uint64_t now) {
    this->firstClock = now;
}

void NaoAudioImporter::processBuffer(GstBuffer *buffer) {
    gst_buffer_ref(buffer);

    // Ugly
    if (!go) {
	Factory &factory = Factory::getInstance();
	listener = factory.createListener(Scope(scopename));
	listener->addHandler(HandlerPtr(new EventQueuePushHandler(queue)));
    }
    go = true;
    try {
	copyBuffer(buffer);
    } catch (rsc::threading::QueueEmptyException &e) {
	GST_BUFFER_FLAG_SET(buffer, GST_BUFFER_FLAG_GAP);
	GST_BUFFER_TIMESTAMP(buffer) = GST_CLOCK_TIME_NONE;
	return;
    }
    if (this->event) {
	if (firstTimestamp == 0) {
	    firstTimestamp = this->inputTimestamp;
	}

	GST_BUFFER_TIMESTAMP(buffer)
	    = this->firstClock
	    + (this->inputTimestamp - firstTimestamp) * 1000ull;

	//std::cout << "[Audio] set timestamp " << GST_BUFFER_TIMESTAMP(buffer) << std::endl;
    }

    gst_buffer_unref(buffer);
}

gint NaoAudioImporter::getIntProperty(const GstBuffer *buffer,
				      const gchar *name) {
    GstStructure *properties = gst_caps_get_structure(GST_BUFFER_CAPS(buffer),
						      0);
    gint value = 0;
    gboolean hasProp = gst_structure_get_int(properties, name, &value);
    if (!hasProp) {
	std::cout << "Property " << name << " does not exists." << std::endl;
    }
    return value;
}

std::string NaoAudioImporter::getMimeType(const GstBuffer *buffer) {
    GstStructure *properties
	= gst_caps_get_structure(GST_BUFFER_CAPS(buffer), 0);
    return std::string(gst_structure_get_name(properties));
}

void NaoAudioImporter::copyBuffer(GstBuffer *buffer) {
    int16_t	 *output = (int16_t*) GST_BUFFER_DATA (buffer);
    unsigned int  count	 = 0;
    while (count < (GST_BUFFER_SIZE(buffer) / sizeof(int16_t))) {
	if (this->inputRemaining == 0) {
	    this->event = queue->pop();
	    this->message = static_pointer_cast<rst::SoundMessage>(this->event->getData());
	    this->inputTimestamp = event->getMetaData().getCreateTime();

	    this->inputPosition = (int16_t*) &*message->data().begin();
	    this->inputRemaining = message->sample_count() * message->channels();
	}
	for (unsigned int j = 0; j < min(2, (int) message->channels()); j++) {
	    *output++ = *this->inputPosition++;
	    this->inputRemaining--;
	    count++;
	}
	this->inputPosition += 2;
	this->inputRemaining -= 2;
    }
    this->inputTimestamp += (boost::uint64_t) (1000000.0 * (1.0 / 48000.0) * (double) (GST_BUFFER_SIZE(buffer) / (2 * sizeof(int16_t))));
}
