/* ============================================================
 *
 * This file is a part of the RSB project
 *
 * Copyright (C) 2010 by Sebastian Wrede <swrede at techfak dot uni-bielefeld dot de>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include "NaoVideoImporter.h"

#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>

#include <boost/thread.hpp>
#include <boost/timer.hpp>

#include <gst/gst.h>

#include <rsb/Factory.h>
#include <rsb/QueuePushHandler.h>
#include <rsb/converter/Repository.h>
#include <rsb/converter/ProtocolBufferConverter.h>

#include "EventQueuePushHandler.h"

using namespace std;
using namespace rsb;
using namespace boost;
using namespace rsb::converter;

// Constructor
RsbVideoImporter::RsbVideoImporter() :
	queue(new rsc::threading::SynchronizedQueue<EventPtr>(1)),
			firstTimestamp(0) {

	shared_ptr<ProtocolBufferConverter<rst::ImageMessage> > converter(
			new ProtocolBufferConverter<rst::ImageMessage> ());
	cout << "Registering converter for wire schema "
			<< converter->getWireSchema() << endl;
	stringConverterRepository()->registerConverter(converter);

	// You can do this right here, but then the Listener won't get the ScopeName in time.
	// See: gstimport.ccp gst_rsb_import_set_property
	/*
	 Factory &factory = Factory::getInstance();
	 listener = factory.createListener(Scope("/foo/bla"));
	 listener->addHandler(HandlerPtr(new QueuePushHandler<GSimage> (queue)));
	 */

	count = 0;
	gettimeofday(&start, 0);
	go = false;
}

// Destructor
RsbVideoImporter::~RsbVideoImporter() {
}

void RsbVideoImporter::setScopeName(const gchar *scope) {
    g_print("Scope: %s \n", scope);
    this->scopename = std::string(scope);
}

void RsbVideoImporter::setTime(uint64_t now) {
    this->firstClock = now;
}

void RsbVideoImporter::processBuffer(GstBuffer *buffer) {
	gst_buffer_ref(buffer);

	// Ugly
	if (!go) {
		Factory &factory = Factory::getInstance();
		listener = factory.createListener(Scope(scopename));
		listener->addHandler(HandlerPtr(new EventQueuePushHandler(queue)));
	}
	go = true;
	try {
	    this->oldEvent = queue->pop();
	} catch (rsc::threading::QueueEmptyException &e) {
	    GST_BUFFER_FLAG_SET(buffer, GST_BUFFER_FLAG_GAP);
	    GST_BUFFER_TIMESTAMP(buffer) = GST_CLOCK_TIME_NONE;
	    return;
	}
	if (oldEvent) {
		if (firstTimestamp == 0) {
			firstTimestamp = this->oldEvent->getMetaData().getCreateTime();
		}
		boost::shared_ptr<rst::ImageMessage> oldImage =
				boost::static_pointer_cast<rst::ImageMessage>(
						oldEvent->getData());
		char* data_ptr = (char*) GST_BUFFER_DATA (buffer);
		assert(oldImage->data().size() == GST_BUFFER_SIZE(buffer));
		copy(oldImage->data().begin(), oldImage->data().end(), data_ptr);

		GST_BUFFER_TIMESTAMP(buffer)
		    = this->firstClock
		    + (oldEvent->getMetaData().getCreateTime() - firstTimestamp) * 1000ull;

	}
	gst_buffer_unref(buffer);
}

gint RsbVideoImporter::getIntProperty(const GstBuffer *buffer,
		const gchar *name) {
	GstStructure *properties = gst_caps_get_structure(GST_BUFFER_CAPS(buffer),
			0);
	gint value = 0;
	gboolean hasProp = gst_structure_get_int(properties, name, &value);
	if (!hasProp) {
		std::cout << "Property " << name << " does not exists." << std::endl;
	}
	return value;

}

std::string RsbVideoImporter::getMimeType(const GstBuffer *buffer) {
	GstStructure *properties = gst_caps_get_structure(GST_BUFFER_CAPS(buffer),
			0);
	return std::string(gst_structure_get_name(properties));
}
