/*
 * GStreamer
 * Copyright (C) 2005 Thomas Vander Stichele <thomas@apestaart.org>
 * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
 * Copyright (C) 2010 Denis Schulze <<dschulze@techfak.uni-bielefeld.de>>
 * Copyright (C) 2011 Florian Lier <<flier@techfak.uni-bielefeld.de>>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Alternatively, the contents of this file may be used under the
 * GNU Lesser General Public License Version 2.1 (the "LGPL"), in
 * which case the following provisions apply instead of the ones
 * mentioned above:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#pragma once

// STD
#include <iostream>
#include <vector>
#include <string>
#include <math.h>
#include <sys/time.h>
#include <stdio.h>

// RSB
#include <rsb/Listener.h>
#include <rsc/threading/SynchronizedQueue.h>

// GSTREAMER
#include <gst/gst.h>

#include <rst/ImageProcessing.pb.h>

class RsbVideoImporter {

public:

    /** Constructor. */
    RsbVideoImporter();

    /** Destructor. */
    ~RsbVideoImporter();

    void processBuffer(GstBuffer *buffer);

    void setScopeName(const gchar *foo);

    void setTime(boost::uint64_t now);
private:

    rsb::ListenerPtr listener;

    int count;

    bool go;

    timeval start, end;

    std::string scopename;

    /** Extracts an integer property from the buffer if it exists. */
    gint getIntProperty(const GstBuffer *buffer, const gchar* name);

    /** Returns the mime type of the data stored in the buffer. */
    std::string getMimeType(const GstBuffer *buffer);

    boost::shared_ptr<rsc::threading::SynchronizedQueue<rsb::EventPtr> > queue;

    rsb::EventPtr oldEvent;
    boost::uint64_t firstTimestamp;
    boost::uint64_t firstClock;
};
